from approvaltests.core.reporter import Reporter
from approvaltests.reporters.python_native_reporter import PythonNativeReporter


class IntroductionReporter(Reporter):
    def report(self, received_path: str, approved_path: str) -> bool:
        print(self.get_text())
        return PythonNativeReporter().report(received_path, approved_path)

    @staticmethod
    def get_text():
        return """
        Welcome to ApprovalTests!
        No DiffReporters have been detected on this system.
        To learn more, visit [Introduction to Reporters](https://github.com/approvals/ApprovalTests.Python/blob/main/docs/tutorial/intro-to-reporters.md)
        """

    def __str__(self):
        return self.__class__.__name__

    __repr__ = __str__



from typing import Iterator, Any, Callable

from approvaltests.utilities.logger.logging_instance import LoggingInstance
from approvaltests.utilities.string_wrapper import StringWrapper


class SimpleLogger:
    _logger = LoggingInstance()

    @staticmethod
    def register_logger(log_method: Callable[[str], None]) -> None:
        SimpleLogger._logger.logger = log_method

    @staticmethod
    def log_to_string() -> StringWrapper:
        return SimpleLogger._logger.log_to_string()

    @staticmethod
    def use_markers() -> Iterator[None]:
        return SimpleLogger._logger.use_markers(additional_stack=1)

    @staticmethod
    def variable(name: str, value: Any, show_types: bool = False) -> None:
        SimpleLogger._logger.variable(name, value, show_types=show_types)

    @staticmethod
    def hour_glass() -> None:
        SimpleLogger._logger.hour_glass()

    @staticmethod
    def event(event_name: str) -> None:
        SimpleLogger._logger.event(event_name)

    @staticmethod
    def show_timestamps(display: bool) -> None:
        SimpleLogger._logger.log_with_timestamps = display

    @staticmethod
    def query(query_text: str) -> None:
        SimpleLogger._logger.query(query_text)

    @staticmethod
    def message(message: str) -> None:
        SimpleLogger._logger.message(message)

    @staticmethod
    def warning(text: str = "", exception: BaseException = None) -> None:
        SimpleLogger._logger.warning(text, exception)

    @staticmethod
    def show_queries(show: bool):
        SimpleLogger._logger.show_queries(show)

    @staticmethod
    def show_all(show: bool):
        SimpleLogger._logger.show_all(show)

    @staticmethod
    def show_messages(show: bool):
        SimpleLogger._logger.show_messages(show)

    @staticmethod
    def show_variables(show: bool):
        SimpleLogger._logger.show_variables(show)

    @staticmethod
    def show_hour_glass(show: bool):
        SimpleLogger._logger.show_hour_glass(show)

    @staticmethod
    def show_markers(show: bool):
        SimpleLogger._logger.show_markers(show)

    @staticmethod
    def show_events(show: bool):
        SimpleLogger._logger.show_events(show)



import inspect
import os
from inspect import FrameInfo
from typing import Optional, Dict, List

from approvaltests.namer.namer_base import NamerBase
from approvaltests.approval_exception import FrameNotFound


class StackFrameNamer(NamerBase):
    directory = ""
    method_name = ""
    class_name = ""

    def __init__(self, extension: Optional[str] = None) -> None:
        NamerBase.__init__(self, extension)
        self.set_for_stack(inspect.stack(1))
        self.config: Dict[str, str] = {}
        self.config_loaded = False

    def set_for_stack(self, caller: List[FrameInfo]) -> None:
        frame = self.get_test_frame(caller)
        stacktrace = caller[frame]
        self.method_name = stacktrace[3]
        self.class_name = self.get_class_name_for_frame(stacktrace)
        self.directory = os.path.dirname(stacktrace[1])

    @staticmethod
    def get_class_name_for_frame(stacktrace: FrameInfo) -> str:
        if "self" not in stacktrace[0].f_locals:
            name = os.path.splitext(os.path.basename(stacktrace[1]))[0]
        else:
            name = f"{stacktrace[0].f_locals['self'].__class__.__name__}"
        return name

    def get_test_frame(self, caller: List[FrameInfo]) -> int:
        tmp_array = []
        for index, frame in enumerate(caller):
            if self.is_test_method(frame):
                tmp_array.append(index)
        if tmp_array:
            return tmp_array[-1]
        message = """Could not find test method/function. Possible reasons could be:
1) approvaltests is not being used inside a test function
2) your test framework is not supported by ApprovalTests (unittest and pytest are currently supported)."""
        raise FrameNotFound(message)

    @staticmethod
    def is_test_method(frame: FrameInfo) -> bool:
        method_name = frame[3]
        locals = frame[0].f_locals
        is_unittest_test = (
            "self" in locals
            and hasattr(locals["self"], "__dict__")
            and "_testMethodName" in vars(locals["self"])
            and method_name != "__call__"
            and method_name != "_callTestMethod"
            and method_name != "run"
        )

        is_pytest_test = method_name.startswith("test_")

        return is_unittest_test or is_pytest_test

    def get_class_name(self) -> str:
        return self.class_name

    def get_method_name(self) -> str:
        return self.method_name

    def get_directory(self) -> str:
        return self.directory

    def config_directory(self) -> str:
        return self.directory

    def get_file_name(self) -> str:
        class_name = "" if (self.class_name is None) else (self.class_name + ".")
        return class_name + self.method_name



from typing import Any, Callable, Optional, Collection


class Storyboard:
    def __init__(self) -> None:
        self.frame_number = 0
        self.story = ""
        self.add_new_line = False

    def add_frame(self, data: Any, title: Optional[str] = None) -> "Storyboard":
        if self.add_new_line:
            self.story += "\n"
            self.add_new_line = False
        if title:
            self.story += f"{title}:\n"
        elif self.frame_number == 0:
            self.story += "Initial:\n"
        else:
            self.story += f"Frame #{self.frame_number}:\n"
        self.story += f"{data}\n\n"
        self.frame_number += 1

        return self

    def add_frames(
        self, number_of_frames: int, function_for_frame: Callable[[int], Any]
    ) -> "Storyboard":
        for number in range(number_of_frames):
            self.add_frame(function_for_frame(number))
        return self

    def __str__(self) -> str:
        return self.story

    def iterate_frames(
        self, data: Collection[Any], number_of_frames: int = -1
    ) -> "Storyboard":
        if number_of_frames == -1:
            try:
                number_of_frames = len(data)
            except:
                raise RuntimeError(
                    "You must pass in the number of frames for this iterable."
                )
        for _, frame in zip(range(number_of_frames), data):
            self.add_frame(frame)
        return self

    def add_description(self, description):
        self.story += f"{description}\n\n"
        return self

    def add_description_with_data(self, description: str, data: Any) -> "Storyboard":
        self.story += f"{description}: {data}\n"
        self.add_new_line = True
        return self


StoryBoard = Storyboard



from abc import ABC, abstractmethod

from approvaltests.core.verify_parameters import VerifyParameters
from approvaltests.core.options import Options


class Verifiable(ABC):
    @abstractmethod
    def get_verify_parameters(self, options: Options) -> VerifyParameters:
        pass
